#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Artem Alexandrov <alexandrov@devexperts.com>'
__copyright__ = 'Copyright 2014 Artem Alexandrov'
__license__   = """Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)"""

# Please install pyst2
import asterisk.manager

def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    host     = item.config['host']
    port = item.config['port']
    username = item.config['username']
    password = item.config['password']
    extension = item.config['extension']
    context  = item.config['context']
 
    gateway  = item.addrs[0]
    number   = item.addrs[1]    
    title    = item.title
    message  = item.message

    try:
        manager = asterisk.manager.Manager()
        manager.connect(host, port)
        response = manager.login(username, password)
        srv.logging.debug("Authentication {}".format(response))
        channel = gateway + number
        channel_vars = {'text': message}
        # originate the call
        response = manager.originate(channel, extension, context=context, priority='1', caller_id=extension, variables=channel_vars)
        srv.logging.info("Call {}".format(response))
        manager.logoff()
    except asterisk.manager.ManagerSocketException as e:
        srv.logging.error("Error connecting to the manage: {}".format(e))
    except asterisk.manager.ManagerAuthException as e:
        srv.logging.error("Error logging in to the manager: {}".format(e))
    except asterisk.manager.ManagerException as e:
        srv.logging.error("Error: {}".format(e))

    finally:
    # remember to clean up
        try:
            manager.close()
        except asterisk.manager.ManagerSocketException:
            pass
    
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Giovanni Angoli <juzam76()gmail.com>'
__copyright__ = 'Copyright 2018 Giovanni Angoli'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

# This is basically the file.py service but declined for websockets, not more than s/file/websocket/g.

import websocket


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)


    # item.config is brought in from the configuration file
    config   = item.config

    # addrs is a list[] associated with a particular target.
    # While it may contain more than one item (e.g. pushover)
    # the `websocket' service carries one only, i.e. a ws:// or wss:// uri
    uri = item.addrs[0].format(**item.data).encode('utf-8')

    # If the incoming payload has been transformed, use that,
    # else the original payload
    text = item.message

    try:
        ws = websocket.WebSocket()
        ws.connect(uri)
        ws.send(text)
        ws.close()
    except Exception as e:
        srv.logging.warning("Cannot write to websocket `%s': %s" % (uri, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Ben Jones <ben.jones12()gmail.com>'
__copyright__ = 'Copyright 2016 Ben Jones'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import requests
from requests.auth import HTTPBasicAuth

try:
    import simplejson as json
except ImportError:
    import json


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    host = item.config['host']
    port = item.config['port']
    username = item.config['username']
    password = item.config['password']

    # optional ca-cert (usually self-signed cert installed by icinga2)
    if 'cacert' in item.config:
        cacert = item.config['cacert']
    else:
        cacert = None

    # e.g. example.com!ping4
    check_host = item.addrs[0]
    check_service = item.addrs[1]
    check_source = item.addrs[2]

    if check_service is None:
        check_type = 'host'
        check_target = check_host
    else:
        check_type = 'service'
        check_target = '{0}!{1}'.format(check_host, check_service)

    if check_source is None:
        check_source = 'mqttwarn'

    payload = {
        'exit_status': item.priority,
        'plugin_output': item.message,
        'check_source': check_source,
        check_type: check_target,
    }

    # Update our payload with any JSON data in the message.
    try:
        payload.update(json.loads(item.message))
    except Exception:
        pass

    # Request parameters
    headers = {
        "Accept": "application/json"
    }

    kwargs = {
        "headers": headers,
        "auth": HTTPBasicAuth(username, password),
        "json": payload
    }

    if cacert:
        kwargs["verify"] = cacert

    try:
        url = "%s:%d/v1/actions/process-check-result" % (host, port)
        r = requests.post(url, **kwargs)
        if r.status_code != requests.codes.ok:
            srv.logging.warning("Invalid response from icinga2 REST API at `%s`: %s" % (host, r.text))
            return False
    except Exception as e:
        srv.logging.warning("Failed to POST request to icinga2 REST API at `%s': %s" % (host, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import io


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    mode = "a"

    # item.config is brought in from the configuration file
    config = item.config

    # addrs is a list[] associated with a particular target.
    # While it may contain more than one item (e.g. pushover)
    # the `file' service carries one only, i.e. a path name
    filename = item.addrs[0].format(**item.data)

    srv.logging.info("Writing to file `%s'" % (filename))

    # If the incoming payload has been transformed, use that,
    # else the original payload
    text = item.message

    if type(config) == dict and 'append_newline' in config and config['append_newline']:
        text += "\n"
    if type(config) == dict and 'overwrite' in config and config['overwrite']:
        mode = "w"

    try:
        f = io.open(filename, mode, encoding='utf-8')
        f.write(text)
        f.close()

    except Exception as e:
        srv.logging.warning("Cannot write to file `%s': %s" % (filename, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import prowlpy


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    apikey = item.addrs[0]
    application = item.addrs[1]

    title = item.get('title', srv.SCRIPTNAME)
    text = item.message
    priority = item.get('priority', 0)

    try:
        p = prowlpy.Prowl(apikey)
        p.post(application=title,
            event=application,
            description=text,
            priority=priority,
            providerkey=None,
            url=None)
    except Exception as e:
        srv.logging.warning("Cannot prowl: %s" % e)
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import time
import socket


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    # item.config is brought in from the configuration file
    config   = item.config

    # addrs is a list[] associated with a particular target.

    try:
        carbon_host, carbon_port = item.addrs
        carbon_port = int(carbon_port)
    except:
        srv.logging.error("Configuration for target `carbon' is incorrect")
        return False

    # If the incoming payload has been transformed, use that,
    # else the original payload
    text = item.message

    try:
        parts = text.split()
    except:
        srv.logging.error("target `carbon': cannot split string")
        return False

    if len(parts) == 1:
        metric_name = item.data.get('topic', 'ohno').replace('/', '.')
        value = parts[0]
        tics = int(time.time())
    else:
        if len(parts) == 2:
            metric_name = parts[0]
            value = parts[1]
            tics = int(time.time())
        else:
            if len(parts) == 3:
                metric_name = parts[0]
                value = parts[1]
                tics = int(parts[2])

    if metric_name.startswith('.'):     # omit dot there caused by useless leading slash in topic
        metric_name = metric_name[1:]
    carbon_msg = "%s %s %d" % (metric_name, value, tics)
    srv.logging.debug("Sending to carbon: %s" % (carbon_msg))
    carbon_msg = carbon_msg + "\n"
    try:
        sock = socket.socket()
        sock.connect((carbon_host, carbon_port))
        sock.sendall(carbon_msg)
        sock.close()
    except Exception as e:
        srv.logging.warning("Cannot send to carbon service %s:%d: %s" % (carbon_host, carbon_port, e))
        return False

    return True
